package gg.now.sdkdemo;

import static gg.now.sdkdemo.billing.BillingManager.BILLING_MANAGER_NOT_INITIALIZED;

import android.app.AlertDialog;
import android.os.Bundle;
import android.os.Looper;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.annotation.StringRes;
import androidx.annotation.UiThread;
import androidx.appcompat.app.AppCompatActivity;

import com.squareup.picasso.BuildConfig;

import java.io.IOException;
import java.util.UUID;

import gg.now.billingclient.api.BillingClient;
import gg.now.sdkdemo.billing.BillingManager;
import gg.now.sdkdemo.billing.BillingProvider;
import gg.now.sdkdemo.skulist.AcquireFragment;
import gg.now.sdkdemo.skulist.row.DiamondsDelegate;
import gg.now.sdkdemo.skulist.row.StarterPackageDelegate;
import gg.now.sdkdemo.skulist.row.SwordDelegate;
import gg.now.sdkdemo.skulist.row.VIPMonthlyDelegate;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

public class BillingActivity extends AppCompatActivity implements BillingProvider {
    private static final String TAG = "sdkdemo";
    public Button button_open_store, button_send_event;
    private ImageView imageView_vip, imageView_sword, imageView_package, imageView_diamonds;

    private Switch switch_use_guest_payment, switch_use_test_in_game_id;
    private TextView state_vip, state_sword, state_package, state_diamonds, tv_quantity_sword, tv_quantity_package, tv_quantity_diamonds;
    private BillingManager mBillingManager;
    private AcquireFragment mAcquireFragment;
    private MainViewController mViewController;
    private View mScreenWait;
    private final String mInGameId = UUID.randomUUID().toString();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        button_open_store = findViewById(R.id.button_open_store);
        mScreenWait = findViewById(R.id.screen_wait);
        imageView_vip = findViewById(R.id.imageView_vip);
        imageView_sword = findViewById(R.id.imageView_sword);
        imageView_package = findViewById(R.id.imageView_package);
        imageView_diamonds = findViewById(R.id.imageView_diamonds);
        state_vip = findViewById(R.id.state_vip);
        state_sword = findViewById(R.id.state_sword);
        state_package = findViewById(R.id.state_package);
        state_diamonds = findViewById(R.id.state_diamonds);
        tv_quantity_sword = findViewById(R.id.tv_quantity_sword);
        tv_quantity_package = findViewById(R.id.tv_quantity_package);
        tv_quantity_diamonds = findViewById(R.id.tv_quantity_diamonds);
        switch_use_guest_payment = findViewById(R.id.switch_guest_payment);
        switch_use_test_in_game_id = findViewById(R.id.switch_use_test_in_game_id);
        button_send_event = findViewById(R.id.button_sdk_event);

        imageView_sword.setOnClickListener(v -> {
            mBillingManager.initiatePurchaseFlow(SwordDelegate.SKU_ID, BillingClient.SkuType.INAPP, "dev payload");
        });
        imageView_package.setOnClickListener(v -> {
            mBillingManager.initiatePurchaseFlow(StarterPackageDelegate.SKU_ID, BillingClient.SkuType.INAPP, "dev payload");
        });
        imageView_diamonds.setOnClickListener(v -> {
            mBillingManager.initiatePurchaseFlow(DiamondsDelegate.SKU_ID, BillingClient.SkuType.INAPP, "dev payload");
        });
        imageView_vip.setOnClickListener(v -> {
            mBillingManager.initiatePurchaseFlow(VIPMonthlyDelegate.SKU_ID, BillingClient.SkuType.SUBS, "dev payload");
        });

        TextView tv_version = findViewById(R.id.tv_version);
        tv_version.setText(BuildConfig.VERSION_NAME);

        setWaitScreen(true);


        // Start the controller and load game data
        mViewController = new MainViewController(BillingActivity.this);
        initBillingManager();
        switch_use_guest_payment.setOnCheckedChangeListener(
                (buttonView, isChecked) -> {
                    initBillingManager();
                    if (!isChecked) {
                        switch_use_test_in_game_id.setVisibility(View.GONE);
                    } else {
                        switch_use_test_in_game_id.setVisibility(View.VISIBLE);
                    }
                });
        switch_use_test_in_game_id.setOnCheckedChangeListener(
                (buttonView, isChecked) -> {
                    initBillingManager();
                });

        button_open_store.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onOpenStoreButtonClicked(v);
            }
        });

        findViewById(R.id.button_exit).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        button_send_event.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mBillingManager.recordGameEvent("demo_event_name", "data", null);
            }
        });

        updateUi();
    }

    @Override
    protected void onResume() {
        super.onResume();
        // Note: We query purchases in onResume() to handle purchases completed while the activity
        // is inactive. For example, this can happen if the activity is destroyed during the
        // purchase flow. This ensures that when the activity is resumed it reflects the user's
        // current purchases.
//        if (mBillingManager != null
//                && mBillingManager.getBillingClientResponseCode() == BillingClient.BillingResponse.OK) {
//            // make sure this run after init.
//            mBillingManager.queryPurchasesAsync();
//        }
    }

    @Override
    public void onDestroy() {
        Log.d(TAG, "Destroying helper.");
        if (mBillingManager != null) {
            mBillingManager.destroy();
        }
        super.onDestroy();
    }

    /**
     * Remove loading spinner and refresh the UI
     */
    public void showRefreshedUi() {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                setWaitScreen(false);
                updateUi();
                if (mAcquireFragment != null) {
                    mAcquireFragment.refreshUI();
                }
            }
        });
    }

    /**
     * Update UI to reflect model
     */
    @UiThread
    private void updateUi() {
        Log.d(TAG, "Updating the UI. Thread: " + Thread.currentThread().getName());
        imageView_vip.setImageResource(isVIPMonthlySubscribed() ? R.drawable.vip_icon : R.drawable.vip_icon_gray);
        state_vip.setText(isVIPMonthlySubscribed() ? "owned" : "not owned");
        imageView_sword.setImageResource(isSwordPurchased() ? R.drawable.sword_icon : R.drawable.sword_icon_gray);
        state_sword.setText(isSwordPurchased() ? "owned" : "not owned");
        tv_quantity_sword.setText(String.valueOf(mViewController.getQuantityOfSwords()));
        imageView_package.setImageResource(isPackagePurchased() ? R.drawable.starter_package_icon : R.drawable.starter_package_icon_gray);
        state_package.setText(isPackagePurchased() ? "owned" : "not owned");
        tv_quantity_package.setText(String.valueOf(mViewController.getQuantityOfPackages()));
        imageView_diamonds.setImageResource(isDiamondsPurchased() ? R.drawable.diamonds_icon : R.drawable.diamonds_icon_gray);
        state_diamonds.setText(isDiamondsPurchased() ? "owned" : "not owned");
        tv_quantity_diamonds.setText(String.valueOf(mViewController.getQuantityOfDiamonds()));
    }

    /**
     * Enables or disables the "please wait" screen.
     */
    private void setWaitScreen(boolean set) {
        mScreenWait.setVisibility(set ? View.VISIBLE : View.GONE);
    }

    public void onOpenStoreButtonClicked(final View arg0) {
        Log.d(TAG, "open store button clicked.");

        if (mAcquireFragment == null) {
            mAcquireFragment = new AcquireFragment();
        }

        if (!isAcquireFragmentShown()) {
            mAcquireFragment.show(getSupportFragmentManager(), "");

            if (mBillingManager != null
                    && mBillingManager.getBillingClientResponseCode()
                    > BILLING_MANAGER_NOT_INITIALIZED) {
                mAcquireFragment.onManagerReady(this);
            }
        }
    }

    void onBillingManagerSetupFinished() {
        setWaitScreen(false);
        if (mAcquireFragment != null) {
            mAcquireFragment.onManagerReady(this);
        }
    }

    public boolean isAcquireFragmentShown() {
        return mAcquireFragment != null && mAcquireFragment.isVisible();
    }

    @Override
    public BillingManager getBillingManager() {
        return mBillingManager;
    }

    @Override
    public boolean isSwordPurchased() {
        return mViewController.isSwordPurchased();
    }

    @Override
    public boolean isPackagePurchased() {
        return mViewController.isPackagePurchased();
    }

    @Override
    public boolean isDiamondsPurchased() {
        return mViewController.isDiamondsPurchased();
    }

    @Override
    public boolean isVIPMonthlySubscribed() {
        return mViewController.isVIPMonthlySubscribed();
    }

    /**
     * Show an alert dialog to the user
     *
     * @param messageId     String id to display inside the alert dialog
     * @param optionalParam Optional attribute for the string
     */
    @UiThread
    void alert(@StringRes int messageId, @Nullable Object optionalParam) {
        if (Looper.getMainLooper().getThread() != Thread.currentThread()) {
            throw new RuntimeException("Dialog could be shown only from the main thread");
        }

        AlertDialog.Builder bld = new AlertDialog.Builder(this);
        bld.setNeutralButton("OK", null);

        if (optionalParam == null) {
            bld.setMessage(messageId);
        } else {
            bld.setMessage(getResources().getString(messageId, optionalParam));
        }

        bld.create().show();
    }


    private void initBillingManager() {
        boolean isGuestPaymentEnabled = switch_use_guest_payment.isChecked();
        boolean isTestInGameIdEnabled = switch_use_test_in_game_id.isChecked();

        if (isGuestPaymentEnabled) {
            if (isTestInGameIdEnabled) {
                mBillingManager = new BillingManager(BillingActivity.this, mViewController.getUpdateListener());
            } else {
                mBillingManager = new BillingManager(BillingActivity.this, mViewController.getUpdateListener());
            }
        } else {
            mBillingManager = new BillingManager(BillingActivity.this, mViewController.getUpdateListener());
        }
    }

}