package gg.now.wallet.sdk.kotlinwalletdemo

import android.os.Bundle
import android.view.LayoutInflater
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.ViewModelProvider
import gg.now.wallet.sdk.ErrorCodes
import gg.now.wallet.sdk.kotlinwalletdemo.databinding.ActivityLoginBinding
import gg.now.wallet.sdk.kotlinwalletdemo.ui.main.LoginFragment
import gg.now.wallet.sdk.kotlinwalletdemo.ui.main.MainMenuFragment
import gg.now.wallet.sdk.kotlinwalletdemo.ui.main.SplashFragment

class MainActivity : AppCompatActivity() {
    private lateinit var binding: ActivityLoginBinding
    private lateinit var viewModel: MainActivityViewModel
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityLoginBinding.inflate(LayoutInflater.from(this))
        setContentView(R.layout.activity_login)
        viewModel = ViewModelProvider(this)[MainActivityViewModel::class.java]
        viewModel.initialiseWalletSdk(this)
        if (savedInstanceState == null) {
            supportFragmentManager.beginTransaction()
                .replace(R.id.container, SplashFragment.newInstance())
                .commitNow()
        }
        viewModel.isConnected.observe(this) {
            if (it && (viewModel.sessionTopic.value == null || viewModel.sessionTopic.value!!.isEmpty())) {
                launchLoginFragment()
            }
        }
        viewModel.isUserLoggedIn.observe(this) {
            if (it) {
                supportFragmentManager.beginTransaction()
                    .replace(R.id.container, MainMenuFragment.newInstance())
                    .commitNow()
            }
        }
        viewModel.error.observe(this) {
            it?.let { error ->
                Toast.makeText(this, error, Toast.LENGTH_SHORT).show()
                when (error) {
                    ErrorCodes.SESSION_EXPIRED -> {
                        launchLoginFragment()
                    }
                }
            }
        }
    }


    private fun launchLoginFragment() {
        supportFragmentManager.beginTransaction()
            .replace(R.id.container, LoginFragment.newInstance())
            .commitNow()
    }


}