package gg.now.wallet.sdk.kotlinwalletdemo.ui.main

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.View.VISIBLE
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import gg.now.wallet.sdk.exception.SocketNotConnectedException
import gg.now.wallet.sdk.exception.WalletSdkNotInitialisedException
import gg.now.wallet.sdk.kotlinwalletdemo.MainActivity
import gg.now.wallet.sdk.kotlinwalletdemo.MainActivityViewModel
import gg.now.wallet.sdk.kotlinwalletdemo.databinding.FragmentMainMenuBinding


class MainMenuFragment : Fragment() {

    companion object {
        fun newInstance() = MainMenuFragment()
    }

    private lateinit var viewModel: MainMenuViewModel
    private lateinit var mainActivityViewModel: MainActivityViewModel
    private lateinit var binding: FragmentMainMenuBinding

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentMainMenuBinding.inflate(layoutInflater)
        binding.progressBarMainMenu.visibility = View.GONE
        binding.buttonBuyCanny.setOnClickListener {
            try {
                binding.progressBarMainMenu.visibility = View.VISIBLE
                mainActivityViewModel.buyToken(100)
            } catch (ex: SocketNotConnectedException) {
                Toast.makeText(
                    activity,
                    "Failed to establish socket connection, Please check your connectivity",
                    Toast.LENGTH_LONG
                )
            } catch (ex: WalletSdkNotInitialisedException) {
                Toast.makeText(
                    activity,
                    "Sdk not initialised",
                    Toast.LENGTH_LONG
                )
            }
        }
        binding.buttonSendTransaction.setOnClickListener {
            try {
                binding.progressBarMainMenu.visibility = View.VISIBLE
                mainActivityViewModel.sendTransaction()
            } catch (ex: SocketNotConnectedException) {
                Toast.makeText(
                    activity,
                    "Failed to establish socket connection, Please check your connectivity",
                    Toast.LENGTH_LONG
                )
            } catch (ex: WalletSdkNotInitialisedException) {
                Toast.makeText(
                    activity,
                    "Sdk not initialised",
                    Toast.LENGTH_LONG
                )
            }
        }
        binding.buttonTxnDetail.setOnClickListener {
            if (mainActivityViewModel.txnHash.value?.isNotEmpty() == true) {
                val intent =
                    Intent(Intent.ACTION_VIEW, Uri.parse(mainActivityViewModel.txnHash.value!!))
                startActivity(intent)
            }
        }
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        viewModel = ViewModelProvider(this)[MainMenuViewModel::class.java]


        mainActivityViewModel =
            ViewModelProvider(activity as MainActivity)[MainActivityViewModel::class.java]


        mainActivityViewModel.userName.observe(this.viewLifecycleOwner) {
            binding.textViewUserName.text = "username: $it"
        }

        mainActivityViewModel.walletAddress.observe(this.viewLifecycleOwner) {
            binding.textViewAddress.text = "Wallet Address: $it"
            if (it != "") {
                mainActivityViewModel.getTokenBalance()
            }


        }

        mainActivityViewModel.cannyBalance.observe(this.viewLifecycleOwner) {
            binding.progressBarMainMenu.visibility = View.GONE
            binding.textViewCannyBalance.text = "CST balance: $it"
        }

        mainActivityViewModel.txnHash.observe(this.viewLifecycleOwner) {
            binding.progressBarMainMenu.visibility = View.GONE
            if (it.isNotEmpty() && it != "") {
                binding.buttonTxnDetail.visibility = VISIBLE
            }
        }
    }


}